@@ -83,7 +83,6 @@ SITES = {
     'missevan'         : 'missevan',
     'mixcloud'         : 'mixcloud',
     'mtv81'            : 'mtv81',
-    'musicplayon'      : 'musicplayon',
     'miaopai'          : 'yixia',
     'naver'            : 'naver',
     '7gogo'            : 'nanagogo',
@@ -107,8 +106,6 @@ SITES = {
     'twimg'            : 'twitter',
     'twitter'          : 'twitter',
     'ucas'             : 'ucas',
-    'videomega'        : 'videomega',
-    'vidto'            : 'vidto',
     'vimeo'            : 'vimeo',
     'wanmen'           : 'wanmen',
     'weibo'            : 'miaopai',
